@@ -1,4 +1,4 @@
-/* $Id: mem.c,v 1.13.4.4 2000/02/13 15:47:58 martin Exp $ */
+/* $Id: mem.c,v 1.29 2004/06/09 18:52:21 bdejean Exp $ */
 
 /* Copyright (C) 1998 Joshua Sled
    This file is part of LibGTop 1.0.
@@ -29,7 +29,9 @@
 
 #include <sys/sysctl.h>
 #include <sys/vmmeter.h>
+#if defined(__NetBSD__)  && (__NetBSD_Version__ < 105020000)
 #include <vm/vm_param.h>
+#endif
 
 #if defined(__NetBSD__)  && (__NetBSD_Version__ >= 104000000)
 #include <uvm/uvm_extern.h>
@@ -74,7 +76,6 @@ static struct nlist nlst [] = {
 };
 
 /* MIB array for sysctl */
-static int mib_length=2;
 #ifdef __bsdi__
 static int mib [] = { CTL_VM, VM_TOTAL };
 #else
@@ -129,7 +130,7 @@ glibtop_get_mem_p (glibtop *server, glibtop_mem *buf)
 	int bufspace;
 
 	glibtop_init_p (server, (1L << GLIBTOP_SYSDEPS_MEM), 0);
-	
+
 	memset (buf, 0, sizeof (glibtop_mem));
 
 	if (server->sysdeps.mem == 0)
@@ -166,7 +167,7 @@ glibtop_get_mem_p (glibtop *server, glibtop_mem *buf)
 		glibtop_warn_io_r (server, "kvm_read (bufspace)");
 		return;
 	}
-  
+
 	/* convert memory stats to Kbytes */
 
 #if defined(__FreeBSD__)
@@ -192,26 +193,26 @@ glibtop_get_mem_p (glibtop *server, glibtop_mem *buf)
 	v_free_count = vmm.v_free_count;
 #endif
 
-	buf->total = (u_int64_t) pagetok (v_total_count) << LOG1024;
-	buf->used  = (u_int64_t) pagetok (v_used_count) << LOG1024;
-	buf->free  = (u_int64_t) pagetok (v_free_count) << LOG1024;
+	buf->total = (guint64) pagetok (v_total_count) << LOG1024;
+	buf->used  = (guint64) pagetok (v_used_count) << LOG1024;
+	buf->free  = (guint64) pagetok (v_free_count) << LOG1024;
 
 #ifdef __FreeBSD__
-	buf->cached = (u_int64_t) pagetok (vmm.v_cache_count) << LOG1024;
+	buf->cached = (guint64) pagetok (vmm.v_cache_count) << LOG1024;
 #endif
 
 #if defined(__NetBSD__)  && (__NetBSD_Version__ >= 104000000)
-	buf->locked = (u_int64_t) pagetok (uvmexp.wired) << LOG1024;
+	buf->locked = (guint64) pagetok (uvmexp.wired) << LOG1024;
 #else
-	buf->locked = (u_int64_t) pagetok (vmm.v_wire_count) << LOG1024;
+	buf->locked = (guint64) pagetok (vmm.v_wire_count) << LOG1024;
 #endif
 
-	buf->shared = (u_int64_t) pagetok (vmt.t_rmshr) << LOG1024;
+	buf->shared = (guint64) pagetok (vmt.t_rmshr) << LOG1024;
 
 #if __FreeBSD__
-	buf->buffer = (u_int64_t) bufspace;
+	buf->buffer = (guint64) bufspace;
 #else
-	buf->buffer = (u_int64_t) pagetok (bufspace) << LOG1024;
+	buf->buffer = (guint64) pagetok (bufspace) << LOG1024;
 #endif
 
 	/* user */
